==> hlog-0.20-rc1.tar.gz: hlog-0.20-rc1/hlog/common/config.c <==
/*
 * Copyright (c) 2020-2021 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include <jeffpc/io.h>
#include <jeffpc/cstr.h>

#include <hlog/qso.h>
#include <hlog/qso_line.h>
#include <hlog-lua/xlua.h>

#include "config.h"

#define DATA_DIR_CONFIG_FNAME	"config.lua"

#define CFG_TEMPLATE_FXN	"fill_template"
#define CFG_QSOLINE		"qsoline"

static void complain_about_type(lua_State *L, const char *name, int exptype)
{
	fprintf(stderr, "Error: '%s' config item must be a %s, %s found\n",
		name,
		lua_typename(L, exptype),
		lua_typename(L, lua_type(L, -1)));
}

static int process_config(lua_State *L, struct qso *template)
{
	char error[128];
	int ret;

	ASSERT_LUA_STACK(L, 1);

	/* use CFG_TEMPLATE_FXN to set up the template */
	switch (lua_getfield(L, -1, CFG_TEMPLATE_FXN)) {
		case LUA_TFUNCTION:
			/* got the functio, use it */
			ret = xlua_qso_call(L, NULL, qso_getref(template),
					    false, error, sizeof(error));
			if (ret) {
				fprintf(stderr, "Error: 'fill_template' failed "
					"to execute: %s\n", error);
				return ret;
			}
			break;
		case LUA_TNIL:
			/* function not defined - nothing to do */
			lua_pop(L, 1);
			break;
		default:
			complain_about_type(L, CFG_TEMPLATE_FXN,
					    LUA_TFUNCTION);
			return -EINVAL;
	}

	ASSERT_LUA_STACK(L, 1);

	/* use CFG_QSOLINE to set up the qso-line */
	switch (lua_getfield(L, -1, CFG_QSOLINE)) {
		case LUA_TTABLE:
			/* got the table, use it */
			qso_line_init_start();

			lua_pushnil(L);
			while (lua_next(L, -2)) {
				/* stack: table, key, value (top) */

				/* it's ok if we can't convert to a string */
				qso_line_init_add(lua_tostring(L, -1));

				/* pop the value */
				lua_pop(L, 1);
			}

			qso_line_init_done();

			/* pop the qsoline table */
			lua_pop(L, 1);
			break;
		case LUA_TNIL:
			/* table not defined - nothing to do */
			lua_pop(L, 1);
			break;
		default:
			complain_about_type(L, CFG_QSOLINE, LUA_TTABLE);
			return -EINVAL;
	}

	ASSERT_LUA_STACK(L, 1);

	return 0;
}

int load_config(int data_dir_fd, struct qso *template)
{
	char error[128];
	lua_State *L;
	int ret;

	/* set up defaults; the config loading may override it */
	qso_line_init_defaults();

	/* load the config; it should return a table with desired settings */
	L = xlua_init_at(data_dir_fd, DATA_DIR_CONFIG_FNAME, 1, error,
			     sizeof(error));
	if (IS_ERR(L)) {
		fprintf(stderr, "Error: failed to read config: %s\n", error);
		return PTR_ERR(L);
	}

	switch (lua_type(L, -1)) {
		case LUA_TTABLE:
			/* got a table - process it */
			ret = process_config(L, template);
			if (ret)
				goto err_lua;
			break;
		case LUA_TNIL:
			/* config not defined - nothing to do */
			break;
		default:
			fprintf(stderr, "Error: config must be a table, "
				"%s found\n",
				lua_typename(L, lua_type(L, -1)));
			ret = -EINVAL;
			goto err_lua;
	}

	ret = 0;

err_lua:
	lua_close(L);

	if (ret)
		fprintf(stderr, "Error: failed to load config: %s\n",
			xstrerror(ret));

	return ret;
}

==> hlog-0.22.tar.gz: hlog-0.22/hlog/common/config.c <==
/*
 * Copyright (c) 2020-2022 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include <jeffpc/io.h>
#include <jeffpc/cstr.h>

#include <hlog/qso.h>
#include <hlog/qso_line.h>
#include <hlog-lua/xlua.h>

#include "config.h"

#define DATA_DIR_CONFIG_FNAME	"config.lua"

#define CFG_TEMPLATE_FXN	"fill_template"
#define CFG_QSOLINE		"qsoline"

static void complain_about_type(lua_State *L, const char *name, int exptype)
{
	fprintf(stderr, "Error: '%s' config item must be a %s, %s found\n",
		name,
		lua_typename(L, exptype),
		lua_typename(L, lua_type(L, -1)));
}

static int process_config(lua_State *L, struct qso *template, const char *mode)
{
	char error[128];
	int ret;

	ASSERT_LUA_STACK(L, 1);

	/* use CFG_TEMPLATE_FXN to set up the template */
	switch (lua_getfield(L, -1, CFG_TEMPLATE_FXN)) {
		case LUA_TFUNCTION:
			/* got the function, use it */
			ret = xlua_pushqso(L, qso_getref(template), false,
					   error, sizeof(error));
			if (ret) {
				fprintf(stderr, "Error: failed to push args "
					"for 'fill_template': %s\n", error);
				return ret;
			}

			lua_pushstring(L, mode);

			ret = xlua_call(L, NULL, 2, 0, error, sizeof(error));
			if (ret) {
				fprintf(stderr, "Error: 'fill_template' failed "
					"to execute: %s\n", error);
				return ret;
			}
			break;
		case LUA_TNIL:
			/* function not defined - nothing to do */
			lua_pop(L, 1);
			break;
		default:
			complain_about_type(L, CFG_TEMPLATE_FXN,
					    LUA_TFUNCTION);
			return -EINVAL;
	}

	ASSERT_LUA_STACK(L, 1);

	/* use CFG_QSOLINE to set up the qso-line */
	switch (lua_getfield(L, -1, CFG_QSOLINE)) {
		case LUA_TTABLE:
			/* got the table, use it */
			qso_line_init_start();

			lua_pushnil(L);
			while (lua_next(L, -2)) {
				/* stack: table, key, value (top) */

				/* it's ok if we can't convert to a string */
				qso_line_init_add(lua_tostring(L, -1));

				/* pop the value */
				lua_pop(L, 1);
			}

			qso_line_init_done();

			/* pop the qsoline table */
			lua_pop(L, 1);
			break;
		case LUA_TNIL:
			/* table not defined - nothing to do */
			lua_pop(L, 1);
			break;
		default:
			complain_about_type(L, CFG_QSOLINE, LUA_TTABLE);
			return -EINVAL;
	}

	ASSERT_LUA_STACK(L, 1);

	return 0;
}

int load_config(int data_dir_fd, struct qso *template, const char *mode)
{
	char error[128];
	lua_State *L;
	int ret;

	/* set up defaults; the config loading may override it */
	qso_line_init_defaults();

	/* load the config; it should return a table with desired settings */
	L = xlua_init_at(data_dir_fd, DATA_DIR_CONFIG_FNAME, 1, error,
			     sizeof(error));
	if (IS_ERR(L)) {
		fprintf(stderr, "Error: failed to read config: %s\n", error);
		return PTR_ERR(L);
	}

	switch (lua_type(L, -1)) {
		case LUA_TTABLE:
			/* got a table - process it */
			ret = process_config(L, template, mode);
			if (ret)
				goto err_lua;
			break;
		case LUA_TNIL:
			/* config not defined - nothing to do */
			break;
		default:
			fprintf(stderr, "Error: config must be a table, "
				"%s found\n",
				lua_typename(L, lua_type(L, -1)));
			ret = -EINVAL;
			goto err_lua;
	}

	ret = 0;

err_lua:
	lua_close(L);

	if (ret)
		fprintf(stderr, "Error: failed to load config: %s\n",
			xstrerror(ret));

	return ret;
}
